Modified 'nl_sock_transact_multiple__' function to send asynchronous
I/O requests to the kernel driver.

Signed-off-by: Sorin Vinturis <svintu...@cloudbasesolutions.com>
Reported-by: Alin Gabriel Serdean <aserd...@cloudbasesolutions.com>
Reported-at: https://github.com/openvswitch/ovs-issues/issues/64
---
 lib/netlink-socket.c | 26 ++++++++++++++++++++++----
 1 file changed, 22 insertions(+), 4 deletions(-)

diff --git a/lib/netlink-socket.c b/lib/netlink-socket.c
index e4f153f..64f93ff 100644
--- a/lib/netlink-socket.c
+++ b/lib/netlink-socket.c
@@ -832,10 +832,28 @@ nl_sock_transact_multiple__(struct nl_sock *sock,
                              ofpbuf_data(txn->request),
                              ofpbuf_size(txn->request),
                              reply_buf, sizeof reply_buf,
-                             &reply_len, NULL)) {
-            /* XXX: Map to a more appropriate error. */
-            error = EINVAL;
-            break;
+                             &reply_len, &sock->overlapped)) {
+            if (GetLastError() != ERROR_IO_PENDING) {
+                /* XXX: Map to a more appropriate error. */
+                VLOG_DBG_RL(&rl, "nl_transact operation failed - %s",
+                            ovs_strerror(error));
+                error = EINVAL;
+                break;
+            } else {
+                /* Operation has been queued and will complete in the future. 
*/
+                /* Wait for the operation to complete before continuing. */
+                if (!GetOverlappedResult(sock->handle,
+                                        &sock->overlapped,
+                                        &reply_len,
+                                        TRUE)) {
+                    /* Operation has completed, but it failed. */
+                    /* XXX: Map to a more appropriate error. */
+                    VLOG_DBG_RL(&rl, "nl_transact asynchronous operation 
failed - %s",
+                                ovs_strerror(error));
+                    error = EINVAL;
+                    break;
+                }
+            }
         }
 
         if (reply_len < sizeof *reply_nlmsg) {
-- 
1.9.0.msysgit.0
_______________________________________________
dev mailing list
dev@openvswitch.org
http://openvswitch.org/mailman/listinfo/dev

Reply via email to